@@ -2,7 +2,7 @@ import copy
 import logging
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from typing import List, Optional, Tuple, TypeVar, Union
+from typing import List, Optional, Tuple, Union
 
 import numpy as np
 import torch
@@ -10,7 +10,12 @@ from torch.utils.data import Dataset
 
 from mohou.encoding_rule import EncodingRule
 from mohou.types import MultiEpisodeChunk, TerminateFlag
-from mohou.utils import AnyT, assert_equal_with_message, assert_seq_list_list_compatible
+from mohou.utils import (
+    AnyT,
+    assert_equal_with_message,
+    assert_seq_list_list_compatible,
+    flatten_lists,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -31,9 +36,24 @@ class SequenceDatasetConfig:
 
 
 @dataclass
-class SequenceDataAugmentor:  # functor
+class SequenceDataAugmentor:
+    covmat: np.ndarray
     config: SequenceDatasetConfig
-    take_diff: bool = True
+
+    @classmethod
+    def from_seqs(
+        cls, seq_list: List[np.ndarray], config: SequenceDatasetConfig, take_diff: bool = True
+    ):
+        """construct augmentor.
+        seq_list: sequence list used to compute covariance matrix
+        take_diff: if True, covaraince is computed using state difference (e.g. {s[t] - s[t-1]}_{1:T})
+        otherwise covariance is computed using s[t]_{1:T} sequence.
+        """
+        if take_diff:
+            covmat = cls.compute_diff_covariance(seq_list)
+        else:
+            covmat = cls.compute_covariance(seq_list)
+        return cls(covmat, config)
 
     @staticmethod
     def compute_covariance(state_seq_list: List[np.ndarray]) -> np.ndarray:
@@ -43,7 +63,6 @@ class SequenceDataAugmentor:  # functor
 
     @staticmethod
     def compute_diff_covariance(state_seq_list: List[np.ndarray]) -> np.ndarray:
-
         state_diff_list = []
         for state_seq in state_seq_list:
             diff = state_seq[1:, :] - state_seq[:-1, :]
@@ -52,52 +71,48 @@ class SequenceDataAugmentor:  # functor
         cov_mat = np.cov(state_diffs.T)
         return cov_mat
 
-    def apply(self, state_seq_list: List[np.ndarray]) -> List[np.ndarray]:
+    def apply(self, seq: np.ndarray) -> List[np.ndarray]:
         """apply augmentation
-        state_seq_list will be randomized.
-        each seq_list in other_seq_list_list will not be randomized. But just augmented so that they are
-        compatible with augmented state_seq_list.
+        seq: 2dim array (n_seqlen, n_dim)
         """
+        assert seq.ndim == 2
+        assert seq.shape[1] == self.covmat.shape[0]
 
-        if self.take_diff:
-            cov_mat = self.compute_diff_covariance(state_seq_list)
-        else:
-            cov_mat = self.compute_covariance(state_seq_list)
-        cov_mat_scaled = cov_mat * self.config.cov_scale**2
+        covmat_scaled = self.covmat * self.config.cov_scale**2
 
-        noised_state_seq_list = []
+        seq_original = copy.deepcopy(seq)
+        auged_seq_list: List[np.ndarray] = [seq_original]
         for _ in range(self.config.n_aug):
-            for state_seq in state_seq_list:
-                n_seq, n_dim = state_seq.shape
-                mean = np.zeros(n_dim)
-                noise_seq = np.random.multivariate_normal(mean, cov_mat_scaled, n_seq)
-                noised_state_seq_list.append(state_seq + noise_seq)
+            n_seqlen, n_dim = seq_original.shape
+            mean = np.zeros(n_dim)
+            noises = np.random.multivariate_normal(mean, covmat_scaled, n_seqlen)
+            noised_seq = seq_original + noises
+            auged_seq_list.append(noised_seq)
 
-        state_seq_list_auged = copy.deepcopy(state_seq_list)
-        state_seq_list_auged.extend(noised_state_seq_list)
-        return state_seq_list_auged
-
-    def apply_norand(self, seq_list: List[AnyT]) -> List[AnyT]:
-        """data augmentation with the same order as apply but without any randomization"""
-        seq_list_auged = copy.deepcopy(seq_list)
-        for _ in range(self.config.n_aug):
-            for seq in seq_list:
-                seq_list_auged.append(copy.deepcopy(seq))
-        return seq_list_auged
+        return auged_seq_list
 
 
-ArrayOrListT = TypeVar("ArrayOrListT", bound=Union[np.ndarray, List])
+@dataclass
+class PaddingSequenceAligner:
+    n_seqlen_target: int
 
+    @classmethod
+    def from_seqs(
+        cls, seqs: Union[List[np.ndarray], List[List]], n_after_termination: int
+    ) -> "PaddingSequenceAligner":
+        n_seqlen_max = max([len(seq) for seq in seqs])
+        n_seqlen_target = n_seqlen_max + n_after_termination
+        return cls(n_seqlen_target)
 
-def make_same_length(seq_list: List[ArrayOrListT], n_after_termination: int) -> List[ArrayOrListT]:
-    n_seqlen_max = max([len(seq) for seq in seq_list])
-    n_seqlen_target = n_seqlen_max + n_after_termination
-
-    seq_list = copy.deepcopy(seq_list)
-    padded_seq_list: List[ArrayOrListT] = []
-    for seq in seq_list:
+    def apply(self, seq: AnyT) -> AnyT:  # TODO: specify type
+        """get padded sequence based on seq.
+        seq: np.ndarray or List
+        if seq is [1, 2, 3, 4] and n_seqlen_target = 10, the output padded sequence looks like
+        [1, 2, 3, 4, 4, 4, 4, 4, 4, 4]
+        """
+        seq = copy.deepcopy(seq)
         n_seqlen = len(seq)
-        n_padding = n_seqlen_target - n_seqlen
+        n_padding = self.n_seqlen_target - n_seqlen
         assert n_padding >= 0
 
         # TODO(HiroIshida) To remove type-ignores it's better to use singledispatch .
@@ -115,9 +130,8 @@ def make_same_length(seq_list: List[ArrayOrListT], n_after_termination: int) ->
         else:
             assert False
 
-        assert len(padded_seq) == n_seqlen_target
-        padded_seq_list.append(padded_seq)  # type: ignore
-    return padded_seq_list
+        assert len(padded_seq) == self.n_seqlen_target
+        return padded_seq  # type: ignore
 
 
 class WeightPolicy(ABC):
@@ -203,20 +217,23 @@ class AutoRegressiveDataset(Dataset):
         )
 
         # augmentation
-        augmentor = SequenceDataAugmentor(augconfig, take_diff=True)
-        state_seq_list_auged = augmentor.apply(state_seq_list)
-        weight_seq_list_auged = augmentor.apply_norand(weight_seq_list)
-        static_context_list_auged = augmentor.apply_norand(static_context_list)
-        assert weight_seq_list_auged is not None  # for mypy
+        augmentor = SequenceDataAugmentor.from_seqs(state_seq_list, augconfig)
+        state_seq_list_auged = flatten_lists([augmentor.apply(seq) for seq in state_seq_list])
+        weight_seq_list_auged = flatten_lists(
+            [[copy.deepcopy(seq) for _ in range(augconfig.n_aug + 1)] for seq in weight_seq_list]
+        )  # +1 for original data
+        static_context_list_auged = flatten_lists(
+            [[copy.deepcopy(c) for _ in range(augconfig.n_aug + 1)] for c in static_context_list]
+        )  # +1 for original data
 
         # make all sequence to the same length due to torch batch computation requirement
         assert_seq_list_list_compatible([state_seq_list_auged, weight_seq_list_auged])
-        state_seq_list_auged_adjusted = make_same_length(
+        aligner = PaddingSequenceAligner.from_seqs(
             state_seq_list_auged, augconfig.n_dummy_after_termination
         )
-        weight_seq_list_auged_adjusted = make_same_length(
-            weight_seq_list_auged, augconfig.n_dummy_after_termination
-        )
+        state_seq_list_auged_adjusted = [aligner.apply(seq) for seq in state_seq_list_auged]
+        weight_seq_list_auged_adjusted = [aligner.apply(seq) for seq in weight_seq_list_auged]
+
         return cls(
             state_seq_list_auged_adjusted,
             static_context_list_auged,
@@ -258,11 +275,11 @@ class MarkovControlSystemDataset(Dataset):
         obs_seq_list = observation_encoding_rule.apply_to_multi_episode_chunk(chunk)
         assert_seq_list_list_compatible([ctrl_seq_list, obs_seq_list])
 
-        ctrl_augmentor = SequenceDataAugmentor(config, take_diff=False)
-        obs_augmentor = SequenceDataAugmentor(config, take_diff=True)
+        ctrl_augmentor = SequenceDataAugmentor.from_seqs(ctrl_seq_list, config, take_diff=False)
+        obs_augmentor = SequenceDataAugmentor.from_seqs(obs_seq_list, config, take_diff=True)
 
-        ctrl_seq_list_auged = ctrl_augmentor.apply(ctrl_seq_list)
-        obs_seq_list_auged = obs_augmentor.apply(obs_seq_list)
+        ctrl_seq_list_auged = flatten_lists([ctrl_augmentor.apply(seq) for seq in ctrl_seq_list])
+        obs_seq_list_auged = flatten_lists([obs_augmentor.apply(seq) for seq in obs_seq_list])
 
         inp_ctrl_seq = []
         inp_obs_seq = []
